@@ -1,9 +1,10 @@
 """
-Functions to unpack Simrad EK60 .raw data file and save to .nc.
+Functions to unpack Simrad EK60 .raw data file and save to netCDF or zarr.
 """
 
 
 import os
+import re
 import shutil
 from collections import defaultdict
 import numpy as np
@@ -11,16 +12,23 @@ import xarray as xr
 from datetime import datetime as dt
 import pytz
 import pynmea2
-
-from echopype.convert.utils.ek_raw_io import RawSimradFile, SimradEOF
-from echopype.convert.utils.nmea_data import NMEAData
-from echopype.convert.utils.set_groups import SetGroups
-from echopype._version import get_versions
+from .._version import get_versions
+from .utils.ek_raw_io import RawSimradFile, SimradEOF
+from .utils.nmea_data import NMEAData
+from .utils.set_groups import SetGroups
 from .convertbase import ConvertBase
 ECHOPYPE_VERSION = get_versions()['version']
 del get_versions
 
 
+# String used to match filename patterns
+#  survey: not currently saved anywhere
+#  date: used in top-level group for variable date_created
+#  time: used in top-level group for variable date_created
+#  postfix: so far only saw Saildrone that produces filename with postfix
+FILENAME_MATCHER_STR = '(?P<survey>.+)?-?D(?P<date>\w{1,8})-T(?P<time>\w{1,6})-?(?P<postfix>\w+)?.raw'
+
+
 # Create a constant to convert indexed power to power.
 INDEX2POWER = (10.0 * np.log10(2.0) / 256.0)
 
@@ -31,7 +39,7 @@ INDEX2ELEC = 180.0 / 128.0
 class ConvertEK60(ConvertBase):
     """Class for converting EK60 ``.raw`` files.
     """
-    def __init__(self, _filename=''):
+    def __init__(self, _filename='', regex=FILENAME_MATCHER_STR, nmea_gps_sentence='GGA'):
         ConvertBase.__init__(self)
         self.filename = _filename  # path to EK60 .raw filename to be parsed
 
@@ -44,13 +52,15 @@ class ConvertEK60(ConvertBase):
         self.ping_time = []    # list to store ping time
         self.CON1_datagram = None    # storage for CON1 datagram for ME70
 
-        # Variables only used in EK60 parsing
+        # Variables used in EK60 parsing
         self.range_lengths = None    # number of range_bin groups
         self.ping_time_split = {}    # dictionaries to store variables of each range_bin groups (if there are multiple)
         self.power_dict_split = {}
         self.angle_dict_split = {}
         self.tx_sig = {}   # dictionary to store transmit signal parameters and sample interval
         self.ping_slices = []
+        self.timestamp_pattern = re.compile(regex)
+        self.nmea_gps_sentence = nmea_gps_sentence  # select GPS datagram in _set_platform_dict()
 
     def _append_channel_ping_data(self, ch_num, datagram):
         """ Append ping-by-ping channel metadata extracted from the newly read datagram of type 'RAW'.
@@ -275,10 +285,11 @@ class ConvertEK60(ConvertBase):
 
     # Functions to set various dictionaries
     def _set_toplevel_dict(self, raw_file):
-        # filename must have "-" as the field separator for the last 2 fields. Uses first file
-        filename_tup = os.path.splitext(os.path.basename(raw_file))[0].split("-")
-        filedate = filename_tup[len(filename_tup) - 2].replace("D", "")
-        filetime = filename_tup[len(filename_tup) - 1].replace("T", "")
+        # filename must have timestamp that matches self.timestamp_pattern
+        raw_date_time = self.timestamp_pattern.match(os.path.basename(raw_file))
+        filedate = raw_date_time['date']
+        filetime = raw_date_time['time']
+
         out_dict = dict(Conventions='CF-1.7, SONAR-netCDF4, ACDD-1.3',
                         keywords='EK60',
                         sonar_convention_authority='ICES',
@@ -332,8 +343,8 @@ class ConvertEK60(ConvertBase):
 
     def _set_platform_dict(self, out_file=None, piece_seq=0):
         out_dict = dict()
-        # TODO: Need to reconcile the logic between using the unpacked "survey_name"
-        #  and the user-supplied platform_name
+        # TODO: platform_name is not part of the .raw file, so should add set method for user to add
+        # TODO: it seems a good idea to add a variable survey_name to the top-level group
         # self.platform_name = self.config_datagram['survey_name']
         out_dict['platform_name'] = self.platform_name
         out_dict['platform_type'] = self.platform_type
@@ -345,17 +356,23 @@ class ConvertEK60(ConvertBase):
         out_dict['pitch'] = np.array(self.ping_data_dict[1]['pitch'], dtype='float32')
         out_dict['roll'] = np.array(self.ping_data_dict[1]['roll'], dtype='float32')
         out_dict['heave'] = np.array(self.ping_data_dict[1]['heave'], dtype='float32')
+        # TODO: need to check and resolve difference if any on water_level for EK60 and EK80
         # water_level is set to 0 for EK60 since this is not separately recorded
         # and is part of transducer_depth
         out_dict['water_level'] = np.int32(0)
 
         # Read lat/long from NMEA datagram
-        idx_loc = np.argwhere(np.isin(self.nmea_data.messages, ['GGA', 'GLL', 'RMC'])).squeeze()
-        # TODO: use NaN when nmea_msg is empty
+        # Only use messages specified in self.nmea_gps_sentence
+        idx_loc = np.argwhere(np.isin(self.nmea_data.messages, self.nmea_gps_sentence)).squeeze()
+        if idx_loc.size == 1:  # in case of only 1 matching message
+            idx_loc = np.expand_dims(idx_loc, axis=0)
+        # Use NaN if any part of nmea_msg is empty
         nmea_msg = []
         [nmea_msg.append(pynmea2.parse(self.nmea_data.raw_datagrams[x])) for x in idx_loc]
-        out_dict['lat'] = np.array([x.latitude for x in nmea_msg])
-        out_dict['lon'] = np.array([x.longitude for x in nmea_msg])
+        out_dict['lat'] = np.array([x.latitude if hasattr(x, 'latitude') else np.nan
+                                    for x in nmea_msg]) if nmea_msg else [np.nan]
+        out_dict['lon'] = np.array([x.longitude if hasattr(x, 'longitude') else np.nan
+                                    for x in nmea_msg]) if nmea_msg else [np.nan]
         out_dict['location_time'] = self.nmea_data.nmea_times[idx_loc]
 
         if len(self.range_lengths) > 1:
@@ -584,11 +601,13 @@ class ConvertEK60(ConvertBase):
                 # Construct a new path with _part[n] if there are multiple range_bin lengths
                 save_path = split[0] + '_part%02d' % (n + 1) + split[1]
             # Open multiple files as one dataset of each group and save them into a single file
+            with xr.open_dataset(file_group[0]) as ds_top:
+                ds_top.to_netcdf(path=save_path, mode='w')
             with xr.open_dataset(file_group[0], group='Provenance') as ds_prov:
-                ds_prov.to_netcdf(path=save_path, mode='w', group='Provenance')
+                ds_prov.to_netcdf(path=save_path, mode='a', group='Provenance')
             with xr.open_dataset(file_group[0], group='Sonar') as ds_sonar:
                 ds_sonar.to_netcdf(path=save_path, mode='a', group='Sonar')
-            with xr.open_mfdataset(file_group, group='Beam', combine='by_coords') as ds_beam:
+            with xr.open_mfdataset(file_group, group='Beam', combine='by_coords', data_vars='minimal') as ds_beam:
                 ds_beam.to_netcdf(path=save_path, mode='a', group='Beam')
             with xr.open_dataset(file_group[0], group='Environment') as ds_env:
                 ds_env.to_netcdf(path=save_path, mode='a', group='Environment')
