@@ -1,24 +1,38 @@
+"""
+Functions to unpack Simrad EK80 .raw data file and save to netCDF or zarr.
+"""
+
 import os
+import re
 import shutil
 from collections import defaultdict
 import numpy as np
 import xarray as xr
+import netCDF4
 from datetime import datetime as dt
 import pytz
 import pynmea2
-from echopype.convert.utils.ek_raw_io import RawSimradFile, SimradEOF
-from echopype.convert.utils.nmea_data import NMEAData
-from echopype.convert.utils.set_groups import SetGroups
-from echopype._version import get_versions
+from .._version import get_versions
+from .utils.ek_raw_io import RawSimradFile, SimradEOF
+from .utils.nmea_data import NMEAData
+from .utils.set_groups import SetGroups
 from .convertbase import ConvertBase
 ECHOPYPE_VERSION = get_versions()['version']
 del get_versions
 
 
+# String used to match filename patterns
+#  survey: not currently saved anywhere
+#  date: used in top-level group for variable date_created
+#  time: used in top-level group for variable date_created
+#  postfix: so far only saw Saildrone that produces filename with postfix
+FILENAME_MATCHER_STR = '(?P<survey>.+)?-?D(?P<date>\w{1,8})-T(?P<time>\w{1,6})-?(?P<postfix>\w+)?.raw'
+
+
 class ConvertEK80(ConvertBase):
     """Class for converting EK80 ``.raw`` files.
     """
-    def __init__(self, _filename=""):
+    def __init__(self, _filename="", regex=FILENAME_MATCHER_STR, nmea_gps_sentence='GGA'):
         ConvertBase.__init__(self)
         self.filename = _filename  # path to EK60 .raw filename to be parsed
 
@@ -29,6 +43,7 @@ class ConvertEK80(ConvertBase):
         self.power_dict = {}    # dictionary to store power data
         self.angle_dict = {}    # dictionary to store angle data
         self.complex_dict = {}  # dictionary to store complex data
+        self.n_complex_dict = {}  # dictionary to store the number of beams in split-beam complex data
         self.ping_time = []     # list to store ping time
         self.environment = {}   # dictionary to store environment data
         self.parameters = defaultdict(dict)   # Dictionary to hold parameter data
@@ -37,6 +52,17 @@ class ConvertEK80(ConvertBase):
         self.fil_df = defaultdict(dict)       # Dictionary to store filter decimation factors
         self.ch_ids = []                      # List of all channel ids
         self.recorded_ch_ids = []
+        self.timestamp_pattern = re.compile(regex)
+        self.nmea_gps_sentence = nmea_gps_sentence  # select GPS datagram in _set_platform_dict()
+        self._water_level = None
+
+    @property
+    def water_level(self):
+        return self._water_level
+
+    @water_level.setter
+    def water_level(self, water_level):
+        self._water_level = water_level
 
     def _read_datagrams(self, fid):
         """
@@ -113,6 +139,8 @@ class ConvertEK80(ConvertBase):
                 self.power_dict[curr_ch_id].append(new_datagram['power'])  # append power data
                 self.angle_dict[curr_ch_id].append(new_datagram['angle'])  # append angle data
                 self.complex_dict[curr_ch_id].append(new_datagram['complex'])  # append complex data
+                if self.n_complex_dict[curr_ch_id] < 0:
+                    self.n_complex_dict[curr_ch_id] = new_datagram['n_complex']  # update n_complex data
 
             # NME datagrams store ancillary data as NMEA-0817 style ASCII data.
             elif new_datagram['type'].startswith("NME"):
@@ -145,7 +173,6 @@ class ConvertEK80(ConvertBase):
             raw filename
         """
         print('%s  converting file: %s' % (dt.now().strftime('%H:%M:%S'), os.path.basename(raw)))
-
         with RawSimradFile(raw, 'r') as fid:
             self.config_datagram = fid.read(1)
             self.config_datagram['timestamp'] = np.datetime64(self.config_datagram['timestamp'], '[ms]')
@@ -160,6 +187,7 @@ class ConvertEK80(ConvertBase):
                 self.power_dict[ch_id] = []
                 self.angle_dict[ch_id] = []
                 self.complex_dict[ch_id] = []
+                self.n_complex_dict[ch_id] = -1
 
                 # Parameters recorded for each frequency for each ping
                 self.parameters[ch_id]['frequency_start'] = []
@@ -203,10 +231,10 @@ class ConvertEK80(ConvertBase):
 
     # Functions to set various dictionaries
     def _set_toplevel_dict(self, raw_file):
-        # filename must have "-" as the field separator for the last 2 fields. Uses first file
-        filename_tup = os.path.splitext(os.path.basename(raw_file))[0].split("-")
-        filedate = filename_tup[len(filename_tup) - 2].replace("D", "")
-        filetime = filename_tup[len(filename_tup) - 1].replace("T", "")
+        # filename must have timestamp that matches self.timestamp_pattern
+        raw_date_time = self.timestamp_pattern.match(os.path.basename(raw_file))
+        filedate = raw_date_time['date']
+        filetime = raw_date_time['time']
 
         out_dict = dict(Conventions='CF-1.7, SONAR-netCDF4, ACDD-1.3',
                         keywords='EK80',
@@ -250,27 +278,41 @@ class ConvertEK80(ConvertBase):
 
     def _set_platform_dict(self):
         out_dict = dict()
-        # TODO: Need to reconcile the logic between using the unpacked "survey_name"
-        #  and the user-supplied platform_name
+        # TODO: platform_name is not part of the .raw file, so should add set method for user to add
+        # TODO: it seems a good idea to add a variable survey_name to the top-level group
         # self.platform_name = self.config_datagram['survey_name']
         out_dict['platform_name'] = self.platform_name
         out_dict['platform_type'] = self.platform_type
         out_dict['platform_code_ICES'] = self.platform_code_ICES
 
         # Read pitch/roll/heave from ping data
-        out_dict['ping_time'] = self.ping_time  # [seconds since 1900-01-01] for xarray.to_netcdf conversion
-        out_dict['pitch'] = np.array(self.mru_data['pitch'])
-        out_dict['roll'] = np.array(self.mru_data['roll'])
-        out_dict['heave'] = np.array(self.mru_data['heave'])
-        out_dict['water_level'] = self.environment['water_level_draft']
+        out_dict['mru_time'] = np.array(self.mru_data.get('timestamp', [np.nan]))  # [seconds since 1900-01-01] for xarray.to_netcdf conversion
+        out_dict['pitch'] = np.array(self.mru_data.get('pitch', [np.nan]))
+        out_dict['roll'] = np.array(self.mru_data.get('roll', [np.nan]))
+        out_dict['heave'] = np.array(self.mru_data.get('heave', [np.nan]))
+        if self.water_level is not None:
+            out_dict['water_level'] = self.water_level
+        elif 'water_level_draft' in self.environment:
+            out_dict['water_level'] = self.environment['water_level_draft']
+        else:
+            out_dict['water_level'] = None
+            print('WARNING: The water_level_draft was not in the file. Value '
+                  'set to None')
 
         # Read lat/long from NMEA datagram
-        idx_loc = np.argwhere(np.isin(self.nmea_data.messages, ['GGA', 'GLL', 'RMC'])).squeeze()
+        # Only use messages specified in self.nmea_gps_sentence
+        idx_loc = np.argwhere(np.isin(self.nmea_data.messages, self.nmea_gps_sentence)).squeeze()
+        if idx_loc.size == 1:  # in case of only 1 matching message
+            idx_loc = np.expand_dims(idx_loc, axis=0)
+        # Use NaN if any part of nmea_msg is empty
         nmea_msg = []
         [nmea_msg.append(pynmea2.parse(self.nmea_data.raw_datagrams[x])) for x in idx_loc]
-        out_dict['lat'] = np.array([x.latitude for x in nmea_msg])
-        out_dict['lon'] = np.array([x.longitude for x in nmea_msg])
+        out_dict['lat'] = np.array([x.latitude if hasattr(x, 'latitude') else np.nan
+                                    for x in nmea_msg]) if nmea_msg else [np.nan]
+        out_dict['lon'] = np.array([x.longitude if hasattr(x, 'longitude') else np.nan
+                                    for x in nmea_msg]) if nmea_msg else [np.nan]
         out_dict['location_time'] = self.nmea_data.nmea_times[idx_loc]
+
         return out_dict
 
     def _set_nmea_dict(self):
@@ -308,17 +350,19 @@ class ConvertEK80(ConvertBase):
         b_r_tmp = {}      # Real part of broadband backscatter
         b_i_tmp = {}      # Imaginary part of b 99-6 raodband backscatter
 
-        # Find largest array in order to pad and stack smaller arrays
-        max_len = 0
+        # Find largest dimensions of array in order to pad and stack smaller arrays
+        max_samples = 0
+        # TODO How to determine if a CW data set is split beam or single beam, and how many splits?
+        max_splits = max([n_c for n_c in self.n_complex_dict.values()]) if bb else 4
         for tx in ch_ids:
             if bb:
-                reshaped = np.array(self.complex_dict[tx]).reshape((ping_num, -1, 4))
+                reshaped = np.array(self.complex_dict[tx]).reshape((ping_num, -1, self.n_complex_dict[tx]))
                 b_r_tmp[tx] = np.real(reshaped)
                 b_i_tmp[tx] = np.imag(reshaped)
-                max_len = b_r_tmp[tx].shape[1] if b_r_tmp[tx].shape[1] > max_len else max_len
+                max_samples = b_r_tmp[tx].shape[1] if b_r_tmp[tx].shape[1] > max_samples else max_samples
             else:
                 b_r_tmp[tx] = np.array(self.power_dict[tx], dtype='float32')
-                max_len = b_r_tmp[tx].shape[1] if b_r_tmp[tx].shape[1] > max_len else max_len
+                max_samples = b_r_tmp[tx].shape[1] if b_r_tmp[tx].shape[1] > max_samples else max_samples
 
         # Loop through each transducer for channel-specific variables
         bm_width = defaultdict(lambda: np.zeros(shape=(tx_num,), dtype='float32'))
@@ -339,21 +383,21 @@ class ConvertEK80(ConvertBase):
         for k, c in self.config_datagram['configuration'].items():
             if k not in ch_ids:
                 continue
-            bm_width['beamwidth_receive_major'][c_seq] = c['beam_width_alongship']
-            bm_width['beamwidth_receive_minor'][c_seq] = c['beam_width_athwartship']
-            bm_width['beamwidth_transmit_major'][c_seq] = c['beam_width_alongship']
-            bm_width['beamwidth_transmit_minor'][c_seq] = c['beam_width_athwartship']
-            bm_dir['beam_direction_x'][c_seq] = c['transducer_alpha_x']
-            bm_dir['beam_direction_y'][c_seq] = c['transducer_alpha_y']
-            bm_dir['beam_direction_z'][c_seq] = c['transducer_alpha_z']
-            bm_angle['angle_offset_alongship'][c_seq] = c['angle_offset_alongship']
-            bm_angle['angle_offset_athwartship'][c_seq] = c['angle_offset_athwartship']
-            bm_angle['angle_sensitivity_alongship'][c_seq] = c['angle_sensitivity_alongship']
-            bm_angle['angle_sensitivity_athwartship'][c_seq] = c['angle_sensitivity_athwartship']
-            tx_pos['transducer_offset_x'][c_seq] = c['transducer_offset_x']
-            tx_pos['transducer_offset_y'][c_seq] = c['transducer_offset_y']
-            tx_pos['transducer_offset_z'][c_seq] = c['transducer_offset_z']
-            beam_dict['equivalent_beam_angle'][c_seq] = c['equivalent_beam_angle']
+            bm_width['beamwidth_receive_major'][c_seq] = c.get('beam_width_alongship', np.nan)
+            bm_width['beamwidth_receive_minor'][c_seq] = c.get('beam_width_athwartship', np.nan)
+            bm_width['beamwidth_transmit_major'][c_seq] = c.get('beam_width_alongship', np.nan)
+            bm_width['beamwidth_transmit_minor'][c_seq] = c.get('beam_width_athwartship', np.nan)
+            bm_dir['beam_direction_x'][c_seq] = c.get('transducer_alpha_x', np.nan)
+            bm_dir['beam_direction_y'][c_seq] = c.get('transducer_alpha_y', np.nan)
+            bm_dir['beam_direction_z'][c_seq] = c.get('transducer_alpha_z', np.nan)
+            bm_angle['angle_offset_alongship'][c_seq] = c.get('angle_offset_alongship', np.nan)
+            bm_angle['angle_offset_athwartship'][c_seq] = c.get('angle_offset_athwartship', np.nan)
+            bm_angle['angle_sensitivity_alongship'][c_seq] = c.get('angle_sensitivity_alongship', np.nan)
+            bm_angle['angle_sensitivity_athwartship'][c_seq] = c.get('angle_sensitivity_athwartship', np.nan)
+            tx_pos['transducer_offset_x'][c_seq] = c.get('transducer_offset_x', np.nan)
+            tx_pos['transducer_offset_y'][c_seq] = c.get('transducer_offset_y', np.nan)
+            tx_pos['transducer_offset_z'][c_seq] = c.get('transducer_offset_z', np.nan)
+            beam_dict['equivalent_beam_angle'][c_seq] = c.get('equivalent_beam_angle', np.nan)
             # TODO: gain is 5 values in test dataset
             beam_dict['gain_correction'][c_seq] = c['gain'][c_seq]
             beam_dict['gpt_software_version'].append(c['transceiver_software_version'])
@@ -363,21 +407,23 @@ class ConvertEK80(ConvertBase):
             # Pad each channel with nan so that they can be stacked
             # Broadband
             if bb:
-                diff = max_len - b_r_tmp[k].shape[1]
-                beam_dict['backscatter_r'].append(np.pad(b_r_tmp[k], ((0, 0), (0, diff), (0, 0)),
-                                                  mode='constant', constant_values=np.nan))
-                beam_dict['backscatter_i'].append(np.pad(b_i_tmp[k], ((0, 0), (0, diff), (0, 0)),
-                                                  mode='constant', constant_values=np.nan))
+                diff_samples = max_samples - b_r_tmp[k].shape[1]
+                diff_splits = max_splits - b_r_tmp[k].shape[2] if b_r_tmp[k].ndim > 2 else max_splits - 1
+                beam_dict['backscatter_r'].append(np.pad(b_r_tmp[k], ((0, 0), (0, diff_samples), (0, diff_splits)),
+                                                    mode='constant', constant_values=np.nan))
+                beam_dict['backscatter_i'].append(np.pad(b_i_tmp[k], ((0, 0), (0, diff_samples), (0, diff_splits)),
+                                                    mode='constant', constant_values=np.nan))
                 beam_dict['frequency_start'].append(self.parameters[k]['frequency_start'])
                 beam_dict['frequency_end'].append(self.parameters[k]['frequency_end'])
             # Continuous wave
             else:
-                diff = max_len - b_r_tmp[k].shape[1]
-                beam_dict['backscatter_r'].append(np.pad(b_r_tmp[k], ((0, 0), (0, diff)),
-                                                         mode='constant', constant_values=np.nan))
+                diff_samples = max_samples - b_r_tmp[k].shape[1]
+                diff_splits = max_splits - b_r_tmp[k].shape[2] if b_r_tmp[k].ndim > 2 else max_splits - 1
+                beam_dict['backscatter_r'].append(np.pad(b_r_tmp[k], ((0, 0), (0, diff_samples)),
+                                                            mode='constant', constant_values=np.nan))
                 beam_dict['angle_dict'].append(np.pad(np.array(self.angle_dict[k], dtype='float32'),
-                                                      ((0, 0), (0, diff), (0, 0)),
-                                                      mode='constant', constant_values=np.nan))
+                                                        ((0, 0), (0, diff_samples), (0, diff_splits)),
+                                                        mode='constant', constant_values=np.nan))
             c_seq += 1
 
         # Stack channels and order axis as: channel, quadrant, ping, range
@@ -390,7 +436,7 @@ class ConvertEK80(ConvertBase):
         else:
             beam_dict['backscatter_r'] = np.stack(beam_dict['backscatter_r'])
             beam_dict['angle_dict'] = np.stack(beam_dict['angle_dict'])
-        beam_dict['range_bin'] = np.arange(max_len)
+        beam_dict['range_bin'] = np.arange(max_samples)
         beam_dict['beam_width'] = bm_width
         beam_dict['beam_direction'] = bm_dir
         beam_dict['beam_angle'] = bm_angle
@@ -432,15 +478,16 @@ class ConvertEK80(ConvertBase):
         # beam_dict['non_quantitative_processing'] = np.array([0, ] * freq.size, dtype='int32')
         # -- sample_time_offset is set to 2 for EK60 data, this value is NOT from sample_data['offset']
         # beam_dict['sample_time_offset'] = np.array([2, ] * freq.size, dtype='int32')
-        pulse_length = 'pulse_duration_fm' if bb else 'pulse_duration'
-        # Gets indices from pulse length table using the transmit_duration_nominal values selected
-        idx = [np.argwhere(np.isclose(tx_sig['transmit_duration_nominal'][i],
-                                      self.config_datagram['configuration'][ch][pulse_length])).squeeze()
-               for i, ch in enumerate(ch_ids)]
-        # Use the indices to select sa_correction values from the sa correction table
-        beam_dict['sa_correction'] = \
-            np.array([x['sa_correction'][y]
-                     for x, y in zip(self.config_datagram['configuration'].values(), np.array(idx))])
+        if not bb:
+            pulse_length = 'pulse_duration'
+            # Gets indices from pulse length table using the transmit_duration_nominal values selected
+            idx = [np.argwhere(np.isclose(tx_sig['transmit_duration_nominal'][i],
+                                          self.config_datagram['configuration'][ch][pulse_length])).squeeze()
+                   for i, ch in enumerate(ch_ids)]
+            # Use the indices to select sa_correction values from the sa correction table
+            beam_dict['sa_correction'] = \
+                np.array([x['sa_correction'][y]
+                         for x, y in zip(self.config_datagram['configuration'].values(), np.array(idx))])
 
         return beam_dict
 
@@ -458,6 +505,7 @@ class ConvertEK80(ConvertBase):
             decimation_factors[f'{ch}_PC_decimation'] = self.fil_df[ch][2]
         out_dict['filter_coefficients'] = coeffs
         out_dict['decimation_factors'] = decimation_factors
+        out_dict['xml'] = self.config_datagram['xml']
 
         return out_dict
 
@@ -548,6 +596,31 @@ class ConvertEK80(ConvertBase):
             self._set_groups(raw_file, out_file, save_settings=save_settings)
 
     def _combine_files(self):
+        def copy_vendor(src_file, trg_file):
+            # Utility function for copying the filter coefficients from one file into another
+            src = netCDF4.Dataset(src_file)
+            trg = netCDF4.Dataset(trg_file, mode='a')
+            ds_vend = src.groups['Vendor']
+            vdr = trg.createGroup('Vendor')
+            complex64 = np.dtype([("real", np.float32), ("imag", np.float32)])
+            complex64_t = vdr.createCompoundType(complex64, "complex64")
+
+            # set decimation values
+            vdr.setncatts({a: ds_vend.getncattr(a) for a in ds_vend.ncattrs()})
+
+            # Create the dimensions of the file
+            for k, v in ds_vend.dimensions.items():
+                vdr.createDimension(k, len(v) if not v.isunlimited() else None)
+
+            # Create the variables in the file
+            for k, v in ds_vend.variables.items():
+                data = np.empty(len(v), complex64)
+                var = vdr.createVariable(k, complex64_t, v.dimensions)
+                var[:] = data
+
+            src.close()
+            trg.close()
+
         # Do nothing if combine_opt is true if there is nothing to combine
         if not self._temp_path:
             return
@@ -569,20 +642,23 @@ class ConvertEK80(ConvertBase):
                     continue
                 save_path = split[0] + '_cw' + split[1] if n == 0 else self.save_path
             # Open multiple files as one dataset of each group and save them into a single file
+            with xr.open_dataset(file_group[0]) as ds_top:
+                ds_top.to_netcdf(path=save_path, mode='w')
             with xr.open_dataset(file_group[0], group='Provenance') as ds_prov:
-                ds_prov.to_netcdf(path=save_path, mode='w', group='Provenance')
+                ds_prov.to_netcdf(path=save_path, mode='a', group='Provenance')
             with xr.open_dataset(file_group[0], group='Sonar') as ds_sonar:
                 ds_sonar.to_netcdf(path=save_path, mode='a', group='Sonar')
-            with xr.open_mfdataset(file_group, group='Beam', combine='by_coords') as ds_beam:
+            with xr.open_mfdataset(file_group, group='Beam', combine='by_coords', data_vars='minimal') as ds_beam:
                 ds_beam.to_netcdf(path=save_path, mode='a', group='Beam')
             with xr.open_dataset(file_group[0], group='Environment') as ds_env:
                 ds_env.to_netcdf(path=save_path, mode='a', group='Environment')
-            with xr.open_mfdataset(file_group, group='Platform', combine='by_coords') as ds_plat:
+            with xr.open_mfdataset(file_group, group='Platform', combine='nested',
+                                   concat_dim='location_time', data_vars='minimal') as ds_plat:
                 ds_plat.to_netcdf(path=save_path, mode='a', group='Platform')
             with xr.open_mfdataset(file_group, group='Platform/NMEA',
                                    combine='nested', concat_dim='time', decode_times=False) as ds_nmea:
                 ds_nmea.to_netcdf(path=save_path, mode='a', group='Platform/NMEA')
-
+            copy_vendor(file_group[0], save_path)
         # Delete temporary folder:
         shutil.rmtree(self._temp_dir)
 
@@ -608,3 +684,20 @@ class ConvertEK80(ConvertBase):
                 self._export_zarr(save_settings, file_idx)
         if combine_opt and file_format == '.nc':
             self._combine_files()
+
+    def export_xml(self):
+        """ Exports the configuration data as an xml file in the same directory as the data files.
+        """
+        def write_str(file):
+            xml_str = self.config_datagram['xml']
+            xml_path = file[:-3] + 'xml'
+            with open(xml_path, 'w') as xml_file:
+                xml_file.write(xml_str)
+
+        if self.config_datagram is not None:
+            write_str(self.filename[-1])
+        else:
+            for filename in self.filename:
+                self.reset_vars('EK80')
+                self.load_ek80_raw(filename)
+                write_str(filename)
