@@ -9,7 +9,7 @@ Contains low-level functions called by ./ek_raw_parsers.py
 from io import BufferedReader, FileIO, SEEK_SET, SEEK_CUR, SEEK_END
 import struct
 import logging
-import echopype.convert.utils.ek_raw_parsers as parsers
+from . import ek_raw_parsers as parsers
 
 __all__ = ['RawSimradFile']
 
